from django.conf import settings
from django.core.mail import get_connection, send_mail
from django.template import Context as _Context
from django.template.loader import get_template


# Context for with statement (will be in Django 1.7)
class Context(_Context):
    def update(self, data):
        super(Context, self).update(data)
        return self

    def __exit__(self, *args):
        self.pop()

    def __enter__(self):
        pass



class TemplateMailer(object):

    template            = None
    template_name       = None
    html_template       = None
    html_template_name  = None
    from_email          = settings.SERVER_EMAIL
    recipient_list      = None
    auth_user           = None
    auth_password       = None
    connection          = None
    fail_silently       = False
    context             = None
    context_class       = Context

    def __init__(self, **kwargs):
        for name in kwargs:
            if hasattr(self, name):
                setattr(self, name, kwargs.pop(name))
        self.kwargs = kwargs
        if not self.template:
            self.template = self.get_template()
        if not self.html_template:
            self.html_template = self.get_html_template()
        if not self.context:
            self.context = self.context_class(kwargs)
        if not self.connection:
            self.connection = get_connection(
                username=self.auth_user,
                password=self.auth_password,
                fail_silently=self.fail_silently
            )

    def get_template(self):
        return get_template(self.template_name)

    def get_html_template(self):
        return self.html_template_name and get_template(self.html_template_name)

    def get_context(self, dictionary):
        context = self.context_class(**self.kwargs)
        context.update(dictionary)
        return context

    def send_mail(self, from_email=None, recipient_list=None, **kwargs):
        with self.context.update(kwargs):
            content = self.template.render(self.context).strip().split('\n', 1)
            html_message = self.html_template and self.html_template.render(self.context)
        subject = content[0]
        try:
            message = content[1]
        except:
            message = ''
        send_mail(
            subject         = subject,
            message         = message,
            from_email      = from_email      or self.from_email,
            recipient_list  = recipient_list  or self.recipient_list,
            connection      = self.connection,
            html_message    = html_message
        )



from cms.plugin_pool import plugin_pool
from django.utils.translation import ugettext as _

from ..models.roles import (
    FilteredLeaderListPlugin, LeaderListPlugin, LeaderPlugin,
)
from .base import LeprikonPluginBase


@plugin_pool.register_plugin
class LeprikonLeaderPlugin(LeprikonPluginBase):
    name    = _('Leader')
    model   = LeaderPlugin
    raw_id_fields = ('leader',)

    def get_render_template(self, context, instance, placeholder):
        return 'leprikon/cms/leader/%s.html' % instance.template


@plugin_pool.register_plugin
class LeprikonLeaderListPlugin(LeprikonPluginBase):
    name    = _('Leader list')
    model   = LeaderListPlugin
    filter_horizontal = ('age_groups', 'groups', 'leaders')

    def get_render_template(self, context, instance, placeholder):
        return 'leprikon/cms/leader_list/%s.html' % instance.template


@plugin_pool.register_plugin
class LeprikonFilteredLeaderListPlugin(LeprikonPluginBase):
    name    = _('Leader list with search form')
    model   = FilteredLeaderListPlugin
    render_template = 'leprikon/cms/leader_list_filtered.html'



from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from ..utils import first_upper


@python_2_unicode_compatible
class AgeGroup(models.Model):
    name    = models.CharField(_('name'), max_length=50)
    order   = models.IntegerField(_('order'), blank=True, default=0)
    require_school = models.BooleanField(_('require school'), default=True)

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('order',)
        verbose_name        = _('age group')
        verbose_name_plural = _('age groups')

    def __str__(self):
        return self.name

    def title(self):
        return first_upper(self.name)



from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _


@python_2_unicode_compatible
class Place(models.Model):
    name    = models.CharField(_('name'), max_length=50)
    place   = models.CharField(_('place'), max_length=50, blank=True, default='')

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('place',)
        verbose_name        = _('place')
        verbose_name_plural = _('places')

    def __str__(self):
        if self.place:
            return '{name}, {place}'.format(name=self.name, place=self.place)
        else:
            return '{name}'.format(name=self.name)



from django import forms
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _

from .form import FormMixin


class LeaderFilterForm(FormMixin, forms.Form):
    q = forms.CharField(label=_('Search term'), required=False)

    def __init__(self, school_year, *args, **kwargs):
        super(LeaderFilterForm, self).__init__(*args, **kwargs)
        self.qs = school_year.leaders.all()

    def get_queryset(self):
        if not self.is_valid():
            return self.qs
        qs = self.qs
        for word in self.cleaned_data['q'].split():
            qs = qs.filter(
                Q(user__first_name__icontains = word) |
                Q(user__last_name__icontains = word) |
                Q(description__icontains = word)
            )
        return qs.distinct()
